package com.tzld.piaoquan.content.understanding.service.impl;

import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.piaoquan.content.understanding.common.enums.ContentTypeEnum;
import com.tzld.piaoquan.content.understanding.common.enums.DataStatusEnum;
import com.tzld.piaoquan.content.understanding.common.enums.ExceptionEnum;
import com.tzld.piaoquan.content.understanding.common.exception.CommonException;
import com.tzld.piaoquan.content.understanding.dao.mapper.CuPipelineMapper;
import com.tzld.piaoquan.content.understanding.dao.mapper.CuTaskMapper;
import com.tzld.piaoquan.content.understanding.model.dto.ContentDTO;
import com.tzld.piaoquan.content.understanding.model.dto.TaskResultDTO;
import com.tzld.piaoquan.content.understanding.model.param.*;
import com.tzld.piaoquan.content.understanding.model.po.CuPipeline;
import com.tzld.piaoquan.content.understanding.model.po.CuPipelineExample;
import com.tzld.piaoquan.content.understanding.model.po.CuTask;
import com.tzld.piaoquan.content.understanding.model.po.CuTaskExample;
import com.tzld.piaoquan.content.understanding.model.vo.ContentUnderstandingResultVO;
import com.tzld.piaoquan.content.understanding.model.vo.TaskResultVO;
import com.tzld.piaoquan.content.understanding.service.ContentService;
import com.tzld.piaoquan.content.understanding.service.LoghubService;
import com.tzld.piaoquan.content.understanding.service.PipelineService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author supeng
 */
@Slf4j
@Service
public class ContentServiceImpl implements ContentService {

    @Autowired
    private LoghubService loghubService;

    @Autowired
    private PipelineService pipelineService;

    @Value("${video.analyse.pipelineId:1}")
    private Long videoAnalysePipelineId;

    @ApolloJsonValue("${content.understand.pipeline.config:{}}")
    private Map<String, Long> configMap;
    @Autowired
    private CuPipelineMapper cuPipelineMapper;
    @Autowired
    private CuTaskMapper cuTaskMapper;

    @Override
    public void analyse(ContentAnalyseParam param) {
        if (Objects.isNull(param) || Objects.isNull(param.getType())) {
            throw new CommonException(ExceptionEnum.PARAMS_INVALID);
        }
        param.setPipelineId(videoAnalysePipelineId);
        pipelineService.execute(param);
    }

    @Override
    public void understand(ContentUnderstandParam param) {
//        if (Objects.isNull(param) || Objects.isNull(param.getContentTypes()) || param.getContentTypes().isEmpty()
//                || Objects.isNull(param.getContentInfo())) {
//            throw new CommonException(ExceptionEnum.PARAMS_INVALID);
//        }
//        //生成jobId
//        String jobId = UUID.randomUUID().toString().replace("-", "") + System.currentTimeMillis();
//        for (Integer contentType : param.getContentTypes()) {
//            Long pipelineId = configMap.get(contentType.toString());
//            if (Objects.isNull(pipelineId)) {
//                //TODO 日志上报 && 记录状态
//                log.info("jobId = {}", jobId);
//                continue;
//            }
//            ContentUnderstandDTO dto = new ContentUnderstandDTO();
//            BeanUtils.copyProperties(param.getContentInfo(), dto);
//            dto.setPipelineId(pipelineId);
//            dto.setContentType(contentType);
//            pipelineService.execute(dto);
//        }
    }

    @Override
    public TaskResultVO submitTasks(SubmitTasksParam param) {
        //参数校验
        if (Objects.isNull(param) || Objects.isNull(param.getContents()) || param.getContents().isEmpty()) {
            throw new CommonException(ExceptionEnum.PARAMS_INVALID);
        }
        List<ContentDTO> contentDTOList = param.getContents();
        //数据校验
        for (ContentDTO contentDTO : contentDTOList) {
            if (Objects.isNull(contentDTO) || Objects.isNull(contentDTO.getPipelineId())
                    || Objects.isNull(contentDTO.getContent()) || Objects.isNull(contentDTO.getVideoId())) {
                throw new CommonException(ExceptionEnum.PARAMS_INVALID, "videoId、pipelineId、content参数不能为空");
            }
            Long pipelineId = contentDTO.getPipelineId();
            CuPipelineExample example = new CuPipelineExample();
            example.createCriteria().andIdEqualTo(pipelineId).andDataStatusEqualTo(DataStatusEnum.VALID.getValue());
            List<CuPipeline> list = cuPipelineMapper.selectByExample(example);
            if (Objects.isNull(list) || list.isEmpty()) {
                throw new CommonException(ExceptionEnum.PARAMS_INVALID, "pipelineId [" + pipelineId + "]无效");
            }
            CuPipeline cuPipeline = list.get(0);
            if (Objects.isNull(cuPipeline) || !ContentTypeEnum.containsValue(cuPipeline.getContentType())) {
                throw new CommonException(ExceptionEnum.PARAMS_INVALID, "pipelineId [" + pipelineId + "]配置异常");
            }
            contentDTO.setContentType(cuPipeline.getContentType());
        }
        //执行任务
        TaskResultVO taskResultVO = new TaskResultVO();
        List<TaskResultDTO> taskResultDTOS = new ArrayList<>();
        for (ContentDTO contentDTO : contentDTOList) {
            TaskResultDTO taskResultDTO = new TaskResultDTO();
            BeanUtils.copyProperties(contentDTO, taskResultDTO);
            String taskId = pipelineService.executeTask(contentDTO);
            taskResultDTO.setTaskId(taskId);
            taskResultDTOS.add(taskResultDTO);
        }
        taskResultVO.setTaskResults(taskResultDTOS);
        return taskResultVO;
    }

    @Override
    public TaskResultVO queryTasks(QueryTasksParam param) {
        //参数校验
        if (Objects.isNull(param) || Objects.isNull(param.getTaskIds()) || param.getTaskIds().isEmpty()) {
            throw new CommonException(ExceptionEnum.PARAMS_INVALID);
        }
        CuTaskExample example = new CuTaskExample();
        example.createCriteria().andTaskIdIn(param.getTaskIds());
        List<CuTask> list = cuTaskMapper.selectByExample(example);
        TaskResultVO taskResultVO = new TaskResultVO();
        if (Objects.isNull(list) || list.isEmpty()) {
            return taskResultVO;
        }
        List<TaskResultDTO> taskResultDTOS = new ArrayList<>();
        for (CuTask task : list) {
            TaskResultDTO taskResultDTO = new TaskResultDTO();
            BeanUtils.copyProperties(task, taskResultDTO);
            taskResultDTOS.add(taskResultDTO);
        }
        taskResultVO.setTaskResults(taskResultDTOS);
        return taskResultVO;
    }

    @Override
    public ContentUnderstandingResultVO getVideoUnderstandingResult(GetVideoUnderstandingResultParam param) {
        return null;
    }
}
